@@ -1,7 +1,7 @@
 #!/usr/bin/python
 
 # JabberBot: A simple jabber/xmpp bot framework
-# Copyright (c) 2007-2009 Thomas Perl <thpinfo.com>
+# Copyright (c) 2007-2010 Thomas Perl <thpinfo.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 #
 
 
+import os
+import re
 import sys
 
 try:
@@ -31,16 +33,17 @@ import traceback
 """A simple jabber/xmpp bot framework"""
 
 __author__ = 'Thomas Perl <thp@thpinfo.com>'
-__version__ = '0.9'
+__version__ = '0.10'
 __website__ = 'http://thpinfo.com/2007/python-jabberbot/'
 __license__ = 'GPLv3 or later'
 
 def botcmd(*args, **kwargs):
     """Decorator for bot command functions"""
 
-    def decorate(func, hidden=False):
+    def decorate(func, hidden=False, name=None):
         setattr(func, '_jabberbot_command', True)
         setattr(func, '_jabberbot_hidden', hidden)
+        setattr(func, '_jabberbot_command_name', name or func.__name__)
         return func
 
     if len(args):
@@ -72,6 +75,7 @@ class JabberBot(object):
         self.commands = {}
         for name, value in inspect.getmembers(self):
             if inspect.ismethod(value) and getattr(value, '_jabberbot_command', False):
+                name = getattr(value, '_jabberbot_command_name')
                 self.debug('Registered command: %s' % name)
                 self.commands[name] = value
 
@@ -142,9 +146,11 @@ class JabberBot(object):
 
         return self.conn
 
-    def join_room(self, room):
+    def join_room(self, room, username=None):
         """Join the specified multi-user chat room"""
-        my_room_JID = "%s/%s" % (room,self.__username)
+        if username is None:
+            username = self.__username.split('@')[0]
+        my_room_JID = '/'.join(room, username)
         self.connect().send(xmpp.Presence(to=my_room_JID))
 
     def quit( self):
@@ -163,9 +169,47 @@ class JabberBot(object):
         """Send an XMPP message"""
         self.connect().send(mess)
 
+    def send_tune(self, song, debug=False):
+        """Set information about the currently played tune
+
+        Song is a dictionary with keys: file, title, artist, album, pos, track,
+        length, uri. For details see <http://xmpp.org/protocols/tune/>.
+        """
+        NS_TUNE = 'http://jabber.org/protocol/tune'
+        iq = xmpp.Iq(typ='set')
+        iq.setFrom(self.jid)
+        iq.pubsub = iq.addChild('pubsub', namespace = xmpp.NS_PUBSUB)
+        iq.pubsub.publish = iq.pubsub.addChild('publish', attrs = { 'node' : NS_TUNE })
+        iq.pubsub.publish.item = iq.pubsub.publish.addChild('item', attrs= { 'id' : 'current' })
+        tune = iq.pubsub.publish.item.addChild('tune')
+        tune.setNamespace(NS_TUNE)
+
+        title = None
+        if song.has_key('title'):
+            title = song['title']
+        elif song.has_key('file'):
+            title = os.path.splitext(os.path.basename(song['file']))[0]
+        if title is not None:
+            tune.addChild('title').addData(title)
+        if song.has_key('artist'):
+            tune.addChild('artist').addData(song['artist'])
+        if song.has_key('album'):
+            tune.addChild('source').addData(song['album'])
+        if song.has_key('pos') and song['pos'] > 0:
+            tune.addChild('track').addData(str(song['pos']))
+        if song.has_key('time'):
+            tune.addChild('length').addData(str(song['time']))
+        if song.has_key('uri'):
+            tune.addChild('uri').addData(song['uri'])
+
+        if debug:
+            print 'Sending tune:', iq.__str__().encode('utf8')
+        self.conn.send(iq)
+
     def send(self, user, text, in_reply_to=None, message_type='chat'):
         """Sends a simple message to the specified user."""
-        mess = xmpp.Message(user, text)
+        mess = self.build_message(text)
+        mess.setTo(user)
 
         if in_reply_to:
             mess.setThread(in_reply_to.getThread())
@@ -182,17 +226,31 @@ class JabberBot(object):
 
     def build_reply(self, mess, text=None, private=False):
         """Build a message for responding to another message.  Message is NOT sent"""
-        if private: 
-            to_user  = mess.getFrom()
-            type = "chat"
+        response = self.build_message(text)
+        if private:
+            response.setTo(mess.getFrom())
+            response.setType('chat')
         else:
-            to_user  = mess.getFrom().getStripped()
-            type = mess.getType()
-        response = xmpp.Message(to_user, text)
+            response.setTo(mess.getFrom().getStripped())
+            response.setType(mess.getType())
         response.setThread(mess.getThread())
-        response.setType(type)
         return response
 
+    def build_message(self, text):
+        """Builds an xhtml message without attributes."""
+        text_plain = re.sub(r'<[^>]+>', '', text)
+        message = xmpp.protocol.Message(body=text_plain)
+        if text_plain != text:
+            html = xmpp.Node('html', {'xmlns': 'http://jabber.org/protocol/xhtml-im'})
+            try:
+                html.addChild(node=xmpp.simplexml.XML2Node("<body xmlns='http://www.w3.org/1999/xhtml'>" + text.encode('utf-8') + "</body>"))
+                message.addChild(node=html)
+            except Exception, e:
+                # Didn't work, incorrect markup or something.
+                # print >> sys.stderr, e, text
+                message = xmpp.protocol.Message(body=text_plain)
+        return message
+
     def get_sender_username(self, mess):
         """Extract the sender's user name from a message""" 
         type = mess.getType()
@@ -334,7 +392,9 @@ class JabberBot(object):
             # doesn't understand or aren't handled by unknown_command().
             default_reply = 'Unknown command: "%s". Type "help" for available commands.<b>blubb!</b>' % cmd
             if type == "groupchat": default_reply = None
-            reply = self.unknown_command( mess, cmd, args) or default_reply
+            reply = self.unknown_command( mess, cmd, args)
+            if reply is None:
+                reply = default_reply
         if reply:
             self.send_simple_reply(mess,reply)
 
